//*****************************************************************************
//*
//*
//*		VirtualNetworkTools.c
//*
//*
//*****************************************************************************
//
//	This library establishes a connection to a virtal network on a windows
//	system. The driver VirtualNetwork.sys must be installed on the computer.
//
//	A source code sample:
//
//		BYTE		aPacket[1514];
//		LPBYTE		pMac;
//		VnetHandle	hVnet;
//		unsigned	uTurn;
//		int			iSize;
//
//			hVnet = VnetOpen();
//		if(!hVnet)
//			{
//			printf("can't open driver (err=%i)\n",VnetLastError());
//			return -1;
//			}
//
//		pMac = VnetMac(hVnet);
//		printf("connected to the port %i\n",VnetPort(hVnet));
//		printf("own mac: %02X-%02X-%02X-%02X-%02X-%02X\n",pMac[0],pMac[1],pMac[2],pMac[3],pMac[4],pMac[5]);
//
//		for(uTurn=0;uTurn<1000;uTurn++)
//			{
//			   iSize = VnetRecv(hVnet,aPacket,sizeof(aPacket));
//			if(iSize<=0)continue;
//
//			printf("receive %04i bytes:  %02X-%02X-%02X-%02X-%02X-%02X  %02X-%02X-%02X-%02X-%02X-%02X  %02X-%02X\n",iSize,
//					aPacket[ 0],
//					aPacket[ 1],
//					aPacket[ 2],
//					aPacket[ 3],
//					aPacket[ 4],
//					aPacket[ 5],
//					aPacket[ 6],
//					aPacket[ 7],
//					aPacket[ 8],
//					aPacket[ 9],
//					aPacket[10],
//					aPacket[11],
//					aPacket[12],
//					aPacket[13],
//					aPacket[14]);
//			}
//
//		VnetClose(hVnet);
//
//		printf("connection closed\n");
//
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<malloc.h>
#include	<windows.h>
#include 	<winioctl.h>

#include	"Driver/VirtualIoctl.h"
#include	"VirtualNetworkTools.h"



typedef	struct
	{
	HANDLE			hCard;							// handle of the network adapter
	HANDLE			hEvent;							// event for the receiving of packets
	unsigned		uPort;							// used port on the virtual hub
	unsigned		uTimeout;						// timeout for the receiving
	unsigned char 	aMac[6];						// mac address of the virtual port
	}VnetData;

static int	iLastError	= 0;						// last error number


//*****************************************************************************
//*
//*		VnetLastError
//*
//*****************************************************************************
//	Returns the last error number
//		    0 =	no error was happend
//		   -1 = can't open the driver of the selected network card
//		   -2 = no more free ports on the virtual hub
//		   -3 = the selected port was used by an other connection
//		   -4 = not enought memory
//		   -5 = the port can't be initiated
//		   -6 = wrong handle for the virtual network
//		   -7 = can't connect to the port of the virtual hub
//		   -8 = can't set the timeout
//		   -9 = can't reset the virtual port
//		  -10 = wrong packet size
//		  -11 = the packet wasn't send
//		  -12 = the driver can't allocate a buffer
//		  -13 = no packet is in the receive buffer
//		  -14 = not supported
//		  -15 = internal error
int VnetLastError()
{
return iLastError;
}


//*****************************************************************************
//*
//*		VnetOpen
//*
//*****************************************************************************
//	Opens a connection to a virtual network
//	uCard			: is the number of the virtual network adapter (0..N)
//	uPort			: is the port nummer on the virtual hub (0..N or VNET_AUTOPORT)
//	pMacAddr		: is the mac address (at 0 the mac address will be generated)
//	uTimeout		: is the timeout for the packer receiving (in milliseconds)
//	iConnect		: connect to the port (same as cable pulgin)
//	iPromiscuous	: enable promiscuous mode (receive al mac addresses)
//	Returns a handle of the virtual network, or NULL if an error occurs.
VnetHandle VnetOpen(unsigned uCard,unsigned uPort,unsigned char *pMacAddr,unsigned uTimeout,int iConnect,int iPromiscuous)
{
char				cName[128];
VnetSetEventData	sSetEvent;
VnetInitData		sInit;
VnetData			sData;
DWORD				dwBytes;
DWORD				dwError;
LPVOID				pHandle;



	sprintf(cName,"\\\\.\\VirtualNet%i",uCard);

       sData.hCard = CreateFile(cName,GENERIC_READ|GENERIC_WRITE,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,FILE_FLAG_NO_BUFFERING,NULL);
    if(sData.hCard==INVALID_HANDLE_VALUE)			// driver not exists on the card
		{
		iLastError = -1;
        return NULL;
		}


	memset(&sInit,0,sizeof(sInit));
	sInit.uPort			=  uPort;					// port number (0...NIC_MAX_CONNECTIONS-1,0xFFFFFFFF=dynamic)
	sInit.ucPromiscuous	= (iPromiscuous)? 1:0;		// receiving mode
	sInit.ucConnected	= (iConnect    )? 1:0;		// connect with the hub port

	if(pMacAddr)									// no automatic mac address
		{
		memcpy(sInit.aMacAddr,pMacAddr,6);
		}

													// initialize port
	if(!DeviceIoControl(sData.hCard,IOCTL_VNET_INIT_PORT,&sInit,sizeof(sInit),&sInit,sizeof(sInit),&dwBytes,NULL))
		{
		dwError = GetLastError();
		CloseHandle(sData.hCard);

		if(dwError==ERROR_FILE_NOT_FOUND)			// no free port on the hub (STATUS_NO_SUCH_DEVICE)
			{
			iLastError = -2;
			}
		else if(dwError==ERROR_BUSY)				// port already used (STATUS_DEVICE_BUSY)
			{
			iLastError = -3;
			}
		else if(dwError==ERROR_NOT_ENOUGH_MEMORY)	// not enought memory (STATUS_NO_MEMORY)
			{
			iLastError = -4;
			}
		else{										// the port can't be initiated
			iLastError = -5;
			}

        return NULL;
		}




	if(uTimeout)									// set event for receive timeout
		{
		if(uTimeout>10000)uTimeout=10000;

		sData.hEvent		= CreateEvent(NULL,TRUE,FALSE,NULL);
		sData.uTimeout		= uTimeout;

		sSetEvent.aDummy[1]	= 0;
		sSetEvent.uPort		= sInit.uPort;
		sSetEvent.hEvent	= sData.hEvent;

		if(!DeviceIoControl(sData.hCard,IOCTL_VNET_SETEVENT,&sSetEvent,sizeof(sSetEvent),&sSetEvent,sizeof(sSetEvent),&dwBytes,NULL))
			{
			CloseHandle(sData.hCard);
			CloseHandle(sData.hEvent);
			iLastError = -8;
			return 0;
			}
		}


	sData.uPort	= sInit.uPort;
	memcpy(sData.aMac,sInit.aMacAddr,6);

	pHandle	= (VnetData*)malloc(sizeof(VnetData));
	memcpy(pHandle,&sData,sizeof(VnetData));
	iLastError = 0;


return (VnetHandle)pHandle;
}


//*****************************************************************************
//*
//*		VnetClose
//*
//*****************************************************************************
//	Closes a connection to a virtual network
//	pHandle			: handle of the virtual network
//	Returns 0 if successful, or an error number below zero.
//		    0 =	no error was happend
//		   -6 = wrong handle for the virtual net
int VnetClose(VnetHandle pHandle)
{
VnetData	   *pData=(VnetData*)pHandle;


	if(!pData)										// check handle
		{
		iLastError = -6;
		return -6;
		}

	CloseHandle(pData->hCard);

	if(pData->hEvent)								// destroy event
		{
		CloseHandle(pData->hEvent);
		pData->uTimeout	= 0;
		pData->hEvent   = 0;
		}

	pData->hCard = INVALID_HANDLE_VALUE;
	free(pData);
	iLastError = 0;


return 0;
}


//*****************************************************************************
//*
//*		VnetConnect
//*
//*****************************************************************************
//	Connect or dissconnect to the virtual network. This means plugin the
//	netwok cable to the virtual port of the hub.
//	pHandle			: handle of the virtual network
//	iConnect		: selects connect (==0) or dissconnect (!=0)
//	Returns 0 if successful, or an error number below zero
//		    0 =	no error was happend
//		   -6 = wrong handle for the virtual net
//		   -7 = can't connect to the port of the virtual hub
int VnetConnect(VnetHandle pHandle,int iConnect)
{
VnetData	   *pData=(VnetData*)pHandle;
VnetConnectData	sConn;
DWORD			dwBytes;



	if(!pData)										// check handle
		{
		iLastError = -6;
		return -6;
		}


	memset(&sConn,0,sizeof(sConn));
	sConn.uPort			= pData->uPort;
	sConn.ucConnected	= (iConnect)? 1:0;

													// (dis)connect the port
	if(!DeviceIoControl(pData->hCard,IOCTL_VNET_CONNECT,&sConn,sizeof(sConn),&sConn,sizeof(sConn),&dwBytes,NULL))
		{
		iLastError = -7;
		return -7;
		}


	iLastError = 0;



return 0;
}

//*****************************************************************************
//*
//*		VnetReset
//*
//*****************************************************************************
//	Resets the receiving ringbuffer of a virtual port.
//	pHandle			: handle of the virtual network
//	Returns 0 if successful, or an error number below zero
//		    0 =	no error was happend
//		   -6 = wrong handle for the virtual net
//		   -9 = can't reset the virtual port
int VnetReset(VnetHandle pHandle)
{
VnetData	   *pData=(VnetData*)pHandle;
VnetResetData	sReset;
DWORD			dwBytes;



	if(!pData)										// check handle
		{
		iLastError = -6;
		return -6;
		}


	memset(&sReset,0,sizeof(sReset));
	sReset.uPort = pData->uPort;

													// reste the port ring buffer
	if(!DeviceIoControl(pData->hCard,IOCTL_VNET_RESET,&sReset,sizeof(sReset),&sReset,sizeof(sReset),&dwBytes,NULL))
		{
		iLastError = -9;
		return -9;
		}


	iLastError = 0;



return 0;
}

//*****************************************************************************
//*
//*		VnetPort
//*
//*****************************************************************************
//	Gets the port number of the used virtual port on the hub.
//	pHandle			: handle of the virtual network
//	Returns the port number or an error number below zero:
//		   >=0 = is the port number
//		    -6 = wrong handle for the virtual net
int VnetPort(VnetHandle pHandle)
{
VnetData	   *pData=(VnetData*)pHandle;


	if(!pData)										// check handle
		{
		iLastError = -6;
		return -6;
		}


return pData->uPort;
}

//*****************************************************************************
//*
//*		VnetSend
//*
//*****************************************************************************
//	Sends a packet over the port from the virtual hub
//	pHandle			: handle of the virtual network
//	pPacket			: pointer to the packet data
//	iSize			: size of the packet (14 to 1514 bytes)
//	Returns 0 if successful, or an error number below zero
//		    0 =	no error was happend
//		   -6 = wrong handle for the virtual net
//		  -10 = wrong packet size
//		  -11 = the packer wasn't send
int VnetSend(VnetHandle pHandle,const void *pPacket,int iSize)
{
VnetData	   *pData=(VnetData*)pHandle;
DWORD			dwBytes;



	if(!pData)										// check handle
		{
		iLastError = -6;
		return -6;
		}

	if(iSize<16 || iSize>1514)						// check packet size
		{
		iLastError = -10;
		return -10;
		}
													// put packet to the driver
	if(!DeviceIoControl(pData->hCard,IOCTL_VNET_WRITE_PACKET,(LPVOID)pPacket,iSize,NULL,0,&dwBytes,NULL))
		{
		iLastError = -11;
		return -11;
		}

	iLastError = 0;


return 0;
}

//*****************************************************************************
//*
//*		VnetRecv
//*
//*****************************************************************************
//	Receives the next packet from the virtual hub
//	pHandle			: handle of the virtual network
//	pPacket			: pointer to the packet buffer
//	iSize			: size of the packet buffer (1514 bytes or more)
//	Returns the size of the received packet, 0 if no packet was received,
//	or an error number below zero:
//			>0 = a packet was received
//		     0 = nothing received
//		    -6 = wrong handle for the virtual net
//		   -10 = wrong packet buffer size
//		   -12 = the driver can't allocate a buffer
//		   -13 = no packet is in the receive buffer
int VnetRecv(VnetHandle pHandle,void *pPacket,int iSize)
{
VnetData	   *pData=(VnetData*)pHandle;
DWORD			dwBytes;
DWORD			dwError;



	if(!pData)										// check handle
		{
		iLastError = -6;
		return -6;
		}

	if(iSize<1514)									// Packet Gre prfen
		{
		iLastError = -10;
		return -10;
		}

	if(pData->uTimeout)								// wait until next packet
		{
		WaitForSingleObject(pData->hEvent,pData->uTimeout);
		}
													// get packet from the driver
	if(!DeviceIoControl(pData->hCard,IOCTL_VNET_READ_PACKET,NULL,0,pPacket,iSize,&dwBytes,NULL))
		{
		dwError = GetLastError();

		if(dwError==ERROR_INVALID_PARAMETER)
			{
			iLastError = -10;
			}
		else if(dwError==ERROR_NO_SYSTEM_RESOURCES)
			{
			iLastError = -12;
			}
		else{
			iLastError = -13;
			}

		return iLastError;
		}

	iLastError = 0;


return dwBytes;
}

//*****************************************************************************
//*
//*		VnetMac
//*
//*****************************************************************************
//	Get the mac address from the network card on the port side.
//	pHandle			: handle of the virtual network
//	Returns a pointer to the mac adress, or NULL if an error occurs.
unsigned char  *VnetMac(VnetHandle pHandle)
{
VnetData	   *pData=(VnetData*)pHandle;



	if(!pData)										// check handle
		{
		iLastError = -6;
		return 0;
		}


return pData->aMac;
}


//*****************************************************************************
//*
//*		VnetVersion
//*
//*****************************************************************************
//	Gets the version of the driver 
//	pHandle			: handle of the virtual network
//	Returns>=0 if successful, or an error number below zero
//		     0 = no error was happend
//		    -6 = wrong handle for the virtual net
//		   -15 = internal error
int VnetVersion(VnetHandle pHandle)
{
VnetData		   *pData=(VnetData*)pHandle;
VnetVersionData		sVersionInfo;
DWORD				dwBytes;



	if(!pData)										// check handle
		{
		iLastError = -6;
		return -6;
		}

													// get driver version 
	if(!DeviceIoControl(pData->hCard,IOCTL_VNET_VERSION,NULL,0,(LPVOID)&sVersionInfo,sizeof(sVersionInfo),&dwBytes,NULL))
		{
		iLastError = -15;
		return -15;
		}


	iLastError = 0;


return sVersionInfo.uVersion;
}
